@@ -15,12 +15,12 @@ Carp::Assert::More - Convenience assertions for common situations
 
 =head1 VERSION
 
-Version 2.3.0
+Version 2.5.0
 
 =cut
 
 BEGIN {
-    $VERSION = '2.3.0';
+    $VERSION = '2.5.0';
     @ISA = qw(Exporter);
     @EXPORT = qw(
         assert_all_keys_in
@@ -28,10 +28,13 @@ BEGIN {
         assert_arrayref
         assert_arrayref_nonempty
         assert_arrayref_of
+        assert_arrayref_all
         assert_cmp
         assert_coderef
+        assert_context_list
         assert_context_nonvoid
         assert_context_scalar
+        assert_context_void
         assert_datetime
         assert_defined
         assert_empty
@@ -911,6 +914,57 @@ sub assert_arrayref_of($$;$) {
 }
 
 
+=head2 assert_arrayref_all( $aref, $sub [, $name] )
+
+Asserts that I<$aref> is reference to an array that has at least one
+element in it. Each element of the array is passed to subroutine I<$sub>
+which is assumed to be an assertion.
+
+For example:
+
+    my $aref_of_counts = get_counts();
+    assert_arrayref_all( $aref, \&assert_positive_integer, 'Counts are positive' );
+
+Whatever is passed as I<$name>, a string saying "Element #N" will be
+appended, where N is the zero-based index of the array.
+
+=cut
+
+sub assert_arrayref_all($$;$) {
+    my $aref = shift;
+    my $sub  = shift;
+    my $name = shift;
+
+    my @why;
+
+    assert_coderef( $sub, 'assert_arrayref_all requires a code reference' );
+
+    if ( ref($aref) eq 'ARRAY' || (Scalar::Util::blessed( $aref ) && $aref->isa( 'ARRAY' )) ) {
+        if ( @{$aref} ) {
+            my $inner_msg = defined($name) ? "$name: " : 'assert_arrayref_all: ';
+            my $n = 0;
+            for my $i ( @{$aref} ) {
+                $sub->( $i, "${inner_msg}Element #$n" );
+                ++$n;
+            }
+        }
+        else {
+            push @why, 'Array contains no elements';
+        }
+    }
+    else {
+        push @why, 'First argument to assert_arrayref_all was not an array';
+    }
+
+    if ( @why ) {
+        require Carp;
+        &Carp::confess( _failure_msg($name), @why );
+    }
+
+    return;
+}
+
+
 =head2 assert_aoh( $ref [, $name ] )
 
 Verifies that C<$array> is an arrayref, and that every element is a hashref.
@@ -1247,7 +1301,7 @@ Given this function:
     sub something {
         ...
 
-        assert_context_scalar();
+        assert_context_nonvoid();
 
         return $important_value;
     }
@@ -1261,15 +1315,60 @@ but this will fail:
 
     something();
 
+If the C<$name> argument is not passed, a default message of "<funcname>
+must not be called in void context" is provided.
+
 =cut
 
 sub assert_context_nonvoid(;$) {
-    my $name = shift;
-
-    my $wantarray = (caller(1))[5];
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
 
     return if defined($wantarray);
 
+    my $name = $_[0] // "$subroutine must not be called in void context";
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
+=head2 assert_context_void( [$name] )
+
+Verifies that the function currently being executed has been called
+in void context.  This is for functions that do not return anything
+meaningful.
+
+Given this function:
+
+    sub something {
+        ...
+
+        assert_context_void();
+
+        return; # No meaningful value.
+    }
+
+These calls to C<something> will fail:
+
+    my $val = something();
+    my @things = something();
+
+but this will pass:
+
+    something();
+
+If the C<$name> argument is not passed, a default message of "<funcname>
+must be called in void context" is provided.
+
+=cut
+
+sub assert_context_void(;$) {
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
+
+    return if not defined($wantarray);
+
+    my $name = $_[0] // "$subroutine must be called in void context";
+
     require Carp;
     &Carp::confess( _failure_msg($name) );
 }
@@ -1300,15 +1399,59 @@ but these will fail:
     something();
     my @things = something();
 
+If the C<$name> argument is not passed, a default message of "<funcname>
+must be called in scalar context" is provided.
+
 =cut
 
 sub assert_context_scalar(;$) {
-    my $name = shift;
-
-    my $wantarray = (caller(1))[5];
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
 
     return if defined($wantarray) && !$wantarray;
 
+    my $name = $_[0] // "$subroutine must be called in scalar context";
+
+    require Carp;
+    &Carp::confess( _failure_msg($name) );
+}
+
+
+=head2 assert_context_list( [$name] )
+
+Verifies that the function currently being executed has been called in
+list context.
+
+Given this function:
+
+    sub something {
+        ...
+
+        assert_context_scalar();
+
+        return @values;
+    }
+
+This call to C<something> will pass:
+
+    my @vals = something();
+
+but these will fail:
+
+    something();
+    my $thing = something();
+
+If the C<$name> argument is not passed, a default message of "<funcname>
+must be called in list context" is provided.
+
+=cut
+
+sub assert_context_list(;$) {
+    my (undef, undef, undef, $subroutine, undef, $wantarray) = caller(1);
+
+    return if $wantarray;
+
+    my $name = shift // "$subroutine must be called in list context";
+
     require Carp;
     &Carp::confess( _failure_msg($name) );
 }
@@ -1345,7 +1488,7 @@ sub _failure_msg {
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2023 Andy Lester
+Copyright 2005-2024 Andy Lester
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the Artistic License version 2.0.
